__all__ = ['ApiAuthError', 'ApiRequester', 'Audit', 'BadRequestError',
           'BulkRequest', 'BulkWhoisApiError', 'BulkWhoisRecord', 'Client',
           'Contact', 'EmptyApiKeyError', 'ErrorMessage', 'FileError',
           'HttpApiError', 'NameServers', 'ParameterError', 'Registrant',
           'RegistryData', 'ResponseCreate', 'ResponseError',
           'ResponseRecords', 'ResponseRequests',
           'UnparsableApiResponseError', 'WhoisRecord']

from .client import Client

from .models.response import BulkRequest, BulkWhoisRecord, ErrorMessage, \
    RegistryData, ResponseCreate, ResponseRecords, ResponseRequests, \
    WhoisRecord

from .net.http import ApiRequester

from .exceptions.error import ApiAuthError, BadRequestError, \
    BulkWhoisApiError, EmptyApiKeyError, FileError, HttpApiError, \
    ParameterError, ResponseError, UnparsableApiResponseError

from whoisapi import Registrant, Contact, Audit, NameServers



from json import loads

import unittest

from bulkwhoisapi import BulkRequest, BulkWhoisRecord, ErrorMessage, \
    RegistryData, ResponseCreate, ResponseRecords, ResponseRequests, \
    WhoisRecord


_json_response_create_ok = '''{
    "noDataAvailable": false,
    "message": "OK",
    "messageCode": 200,
    "invalidDomains": [
        "foo.bar"
    ],
    "requestId": "12345678-1234-1234-1234-123456789012",
    "whoisRecords": [],
    "domains": []
}'''

_json_response_records_ok = '''{
    "noDataAvailable": false,
    "domainList": [
        "foo.bar"
    ],
    "requestId": "12345678-1234-1234-1234-123456789012",
    "whoisRecords": [
        {
            "domainName": "foo.bar",
            "domainStatus": "I",
            "whoisRecordStatus": 0,
            "domainFetchedTime": "1642158864782",
            "whoisRecord": {
                "registrant": {
                    "rawText": "...",
                    "organization": "foo",
                    "state": "CA",
                    "country": "UNITED STATES",
                    "parseCode": 1
                },
                "administrativeContact": {
                    "rawText": "...",
                    "organization": "foo",
                    "state": "CA",
                    "country": "UNITED STATES",
                    "parseCode": 1
                },
                "technicalContact": {
                    "rawText": "...",
                    "organization": "foo",
                    "state": "CA",
                    "country": "UNITED STATES",
                    "parseCode": 1
                },
                "audit": {},
                "nameServers": {
                    "rawText": "ns1.foo.bar",
                    "hostNames": [
                        {
                            "numeric": false,
                            "str": "ns1.foo.bar"
                        }
                    ]
                },
                "createdDate": "1997-09-15T07:00:00+0000",
                "updatedDate": "2019-09-09T15:39:04+0000",
                "expiresDate": "2028-09-13T07:00:00+0000",
                "domainName": "foo.bar",
                "status": "clientUpdateProhibited",
                "rawText": "...",
                "header": "",
                "strippedText": "...",
                "footer": "",
                "customField1Name": "RegistrarContactEmail",
                "customField1Value": "foo@bar.baz",
                "registrarName": "bar",
                "registrarIANAID": "111111111111111",
                "whoisServer": "foo.bar.baz",
                "createdDateNormalized": "1997-09-15 07:00:00 UTC",
                "updatedDateNormalized": "2019-09-09 15:39:04 UTC",
                "expiresDateNormalized": "2028-09-13 07:00:00 UTC",
                "dataErrorFlag": 0,
                "parseCode": 3579,
                "registryData": {
                    "audit": {},
                    "nameServers": {
                        "rawText": "ns1.foo.bar",
                        "hostNames": [
                            {
                                "numeric": false,
                                "str": "ns1.foo.bar"
                            }
                        ]
                    },
                    "createdDate": "1997-09-15T04:00:00Z",
                    "updatedDate": "2019-09-09T15:39:04Z",
                    "expiresDate": "2028-09-14T04:00:00Z",
                    "domainName": "foo.bar",
                    "status": "clientDeleteProhibited",
                    "rawText": "...",
                    "header": "",
                    "strippedText": "...",
                    "footer": "",
                    "customField1Name": "RegistrarContactEmail",
                    "customField1Value": "foo@bar.baz",
                    "registrarName": "bar",
                    "registrarIANAID": "1111111111111",
                    "whoisServer": "foo.bar.baz",
                    "createdDateNormalized": "1997-09-15 04:00:00 UTC",
                    "updatedDateNormalized": "2019-09-09 15:39:04 UTC",
                    "expiresDateNormalized": "2028-09-14 04:00:00 UTC",
                    "dataErrorFlag": 0,
                    "parseCode": 251
                },
                "contactEmail": "foo@bar.baz",
                "domainNameExt": ".bar",
                "estimatedDomainAge": 8887
            },
            "index": 1
        }
    ],
    "totalRecords": 1,
    "recordsLeft": 0,
    "recordsProcessed": 1
}'''

_json_response_requests_ok = '''{
    "userRequests": [
        {
            "requestId": "12345678-1234-1234-1234-123456789012",
            "time": 1641985855887,
            "status": "Completed",
            "totalRecords": 2,
            "fetchedRecords": 0
        },
        {
            "requestId": "12345678-1234-1234-1234-123456789013",
            "time": 1611308168576,
            "status": "Completed",
            "totalRecords": 1,
            "fetchedRecords": 0
        }
    ]
}'''

_json_response_error = '''{
    "message": "Domain list can not be empty!",
    "messageCode": 417
}'''


class TestModel(unittest.TestCase):

    def test_response_create_parsing(self):
        response = loads(_json_response_create_ok)
        parsed = ResponseCreate(response)
        self.assertEqual(parsed.request_id, response['requestId'])
        self.assertIsInstance(parsed.invalid_domains, list)

        self.assertEqual(parsed.invalid_domains[0],
                         response['invalidDomains'][0])

    def test_response_records_parsing(self):
        response = loads(_json_response_records_ok)
        parsed = ResponseRecords(response)
        self.assertEqual(parsed.request_id, response['requestId'])
        self.assertIsInstance(parsed.whois_records, list)
        self.assertIsInstance(parsed.whois_records[0], BulkWhoisRecord)

        self.assertIsInstance(parsed.whois_records[0].whois_record,
                              WhoisRecord)

        self.assertIsInstance(
            parsed.whois_records[0].whois_record.registry_data,
            RegistryData
        )

        self.assertEqual(
            parsed.whois_records[0].whois_record.registry_data.domain_name,
            response['whoisRecords'][0]
                    ['whoisRecord']['registryData']['domainName']
        )

        self.assertEqual(
            parsed.whois_records[0].whois_record.registrant.organization,
            response['whoisRecords'][0]
                    ['whoisRecord']['registrant']['organization']
        )

    def test_response_requests_parsing(self):
        response = loads(_json_response_requests_ok)
        parsed = ResponseRequests(response)
        self.assertIsInstance(parsed.user_requests, list)
        self.assertIsInstance(parsed.user_requests[0], BulkRequest)

        self.assertEqual(len(parsed.user_requests),
                         len(response['userRequests']))

        self.assertEqual(parsed.user_requests[1].request_id,
                         response['userRequests'][1]['requestId'])

    def test_error_parsing(self):
        error = loads(_json_response_error)
        parsed_error = ErrorMessage(error)
        self.assertEqual(parsed_error.code, error['messageCode'])
        self.assertEqual(parsed_error.message, error['message'])



from json import loads

from ..models.response import ErrorMessage


class BulkWhoisApiError(Exception):
    def __init__(self, message):
        self.message = message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, message):
        self._message = message

    def __str__(self):
        return str(self.__dict__)


class ResponseError(BulkWhoisApiError):
    def __init__(self, message):
        self.message = message
        self.parsed_message = None
        try:
            parsed = loads(message)
            self.parsed_message = ErrorMessage(parsed)
        except Exception:
            pass

    @property
    def parsed_message(self):
        return self._parsed_message

    @parsed_message.setter
    def parsed_message(self, pm):
        self._parsed_message = pm


class ApiAuthError(ResponseError):
    def __init__(self, message):
        super().__init__(message)

        if self.parsed_message.code < 0:
            self.parsed_message.code = 403
            self.parsed_message.message = \
                'Access restricted. Check credits balance ' \
                'or enter a correct API key'


class BadRequestError(ResponseError):
    pass


class EmptyApiKeyError(BulkWhoisApiError):
    pass


class FileError(BulkWhoisApiError):
    pass


class HttpApiError(BulkWhoisApiError):
    pass


class ParameterError(BulkWhoisApiError):
    pass


class UnparsableApiResponseError(BulkWhoisApiError):
    def __init__(self, message, origin_error):
        self.message = message
        self.original_error = origin_error

    @property
    def original_error(self):
        return self._original_error

    @original_error.setter
    def original_error(self, oe):
        self._original_error = oe



from json import loads, JSONDecodeError
from uuid import UUID

import re

from .exceptions.error import EmptyApiKeyError, FileError, ParameterError, \
    UnparsableApiResponseError
from .models.response import ResponseCreate, ResponseRecords, ResponseRequests
from .net.http import ApiRequester


class Client:
    __default_url = 'https://www.whoisxmlapi.com/BulkWhoisLookup/bulkServices'
    _api_requester: ApiRequester or None
    _api_key: str

    _re_api_key = re.compile(r'^at_[a-z0-9]{29}$', re.IGNORECASE)

    _PARSABLE_FORMAT = 'json'

    _PATH_CREATE = '/bulkWhois'
    _PATH_DOWNLOAD = '/download'
    _PATH_REQUESTS = '/getUserRequests'
    _PATH_RECORDS = '/getRecords'

    JSON_FORMAT = 'json'
    XML_FORMAT = 'xml'

    SEARCH_ALL = 'all'
    SEARCH_NO_ERROR = 'noerror'

    def __init__(self, api_key: str, **kwargs):
        """
        :param api_key: str: Your API key
        :key base_url: str: (optional) API endpoint URL
        :key timeout: float: (optional) API call timeout in seconds
        """

        self._api_key = ''

        self.api_key = api_key

        if 'base_url' not in kwargs:
            kwargs['base_url'] = Client.__default_url

        self.api_requester = ApiRequester(**kwargs)

    @property
    def api_key(self) -> str:
        return self._api_key

    @api_key.setter
    def api_key(self, value: str):
        self._api_key = Client._validate_api_key(value)

    @property
    def api_requester(self) -> ApiRequester or None:
        return self._api_requester

    @api_requester.setter
    def api_requester(self, value: ApiRequester):
        self._api_requester = value

    @property
    def base_url(self) -> str:
        return self._api_requester.base_url

    @base_url.setter
    def base_url(self, value: str or None):
        if value is None:
            self._api_requester.base_url = Client.__default_url
        else:
            self._api_requester.base_url = value

    @property
    def timeout(self) -> float:
        return self._api_requester.timeout

    @timeout.setter
    def timeout(self, value: float):
        self._api_requester.timeout = value

    def create_request(self, **kwargs) -> ResponseCreate:
        """
        Create bulk domain names processing request
        :key domains: Required. list[str]
        :return: `ResponseCreate` instance
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        kwargs['output_format'] = Client._PARSABLE_FORMAT

        response = self.create_request_raw(**kwargs)

        try:
            parsed = loads(str(response))
            if 'requestId' in parsed:
                return ResponseCreate(parsed)
            raise UnparsableApiResponseError(
                "Could not find the correct root element.", None)
        except JSONDecodeError as error:
            raise UnparsableApiResponseError(
                    "Could not parse API response",
                    error)

    def download(self, **kwargs):
        """
        Download processing results CSV and save to file
        :key request_id: Required. str. Request ID
        :key search_type: Optional.
                Supported options: SEARCH_ALL, SEARCH_NO_ERROR.
                SEARCH_ALL by default
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        filename = None

        kwargs['output_format'] = Client._PARSABLE_FORMAT

        if 'filename' in kwargs:
            filename = kwargs['filename']

        if type(filename) is not str or not filename:
            raise ParameterError('Output file name required')

        try:
            result_file = open(filename, 'w')
        except Exception:
            raise FileError('Cannot open output file')

        result_file.close()

        response = self.download_raw(**kwargs)

        try:
            result_file = open(filename, 'w')
            result_file.write(response)
        except Exception:
            raise FileError('Cannot write result to file')
        finally:
            result_file.close()

    def get_records(self, **kwargs) -> ResponseRecords:
        """
        Get Whois records
        :key request_id: Required. str. Request ID
        :key max_records: Required. int. Max number of records to return.
                Min: 1
        :key start_index: Optional. int. First record to be returned.
                Min: 1. Use for pagination
        :return: `ResponseRecords` instance
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        kwargs['output_format'] = Client._PARSABLE_FORMAT

        response = self.get_records_raw(**kwargs)

        try:
            parsed = loads(str(response))
            if 'whoisRecords' in parsed:
                return ResponseRecords(parsed)
            raise UnparsableApiResponseError(
                'Cannot find the correct root element', None)
        except JSONDecodeError as error:
            raise UnparsableApiResponseError(
                    'Could not parse API response',
                    error)

    def get_requests(self, **kwargs) -> ResponseRequests:
        """
        Get a list of your requests
        :return: `ResponseRequests` instance
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        kwargs['output_format'] = Client._PARSABLE_FORMAT

        response = self.get_requests_raw(**kwargs)

        try:
            parsed = loads(str(response))
            if 'userRequests' in parsed:
                return ResponseRequests(parsed)
            raise UnparsableApiResponseError(
                'Cannot find the correct root element', None)
        except JSONDecodeError as error:
            raise UnparsableApiResponseError(
                    'Could not parse API response',
                    error)

    def create_request_raw(self, **kwargs) -> str:
        """
        Get raw create response
        :key domains: Required. list[str]
        :key output_format: Optional. Response output format.
                Supported options: JSON_FORMAT, XML_FORMAT.
                JSON_FORMAT by default
        :return: str
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        domains = None

        if self.api_key == '':
            raise EmptyApiKeyError('')

        if 'domains' in kwargs:
            domains = Client._validate_domains(kwargs['domains'])

        if not domains:
            raise ParameterError('Domain names required')

        if 'response_format' in kwargs:
            kwargs['output_format'] = kwargs['response_format']
        if 'output_format' in kwargs:
            output_format = Client._validate_output_format(
                kwargs['output_format'])
        else:
            output_format = Client._PARSABLE_FORMAT

        return self._api_requester.post(
            self._PATH_CREATE,
            self._build_payload(self.api_key, output_format, domains)
        )

    def download_raw(self, **kwargs) -> str:
        """
        Get raw download response
        :key request_id: Required. str. Request ID
        :key search_type: Optional.
                Supported options: SEARCH_ALL, SEARCH_NO_ERROR.
                SEARCH_ALL by default
        :return: str
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        request_id, search_type = [None] * 2

        if self.api_key == '':
            raise EmptyApiKeyError('')

        if 'request_id' in kwargs:
            request_id = Client._validate_request_id(kwargs['request_id'])

        if not request_id:
            raise ParameterError('Request ID required')

        if 'search_type' in kwargs:
            search_type = Client._validate_search_type(kwargs['search_type'])

        return self._api_requester.post(
            self._PATH_DOWNLOAD,
            self._build_payload(
                api_key=self.api_key,
                request_id=request_id,
                search_type=search_type
            )
        )

    def get_records_raw(self, **kwargs) -> str:
        """
        Get raw records response
        :key request_id: Required. str. Request ID
        :key max_records: Required. int. Max number of records to return.
                Min: 1
        :key start_index: Optional. int. First record to be returned.
                Min: 1. Use for pagination
        :key output_format: Optional. Response output format.
                Supported options: JSON_FORMAT, XML_FORMAT.
                JSON_FORMAT by default
        :return: str
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        request_id, max_records, start_index = [None] * 3

        if self.api_key == '':
            raise EmptyApiKeyError('')

        if 'request_id' in kwargs:
            request_id = Client._validate_request_id(kwargs['request_id'])

        if not request_id:
            raise ParameterError('Request ID required')

        if 'max_records' in kwargs:
            max_records = Client._validate_max_records(kwargs['max_records'])

        if not max_records:
            raise ParameterError('Max record number required')

        if 'start_index' in kwargs:
            start_index = Client._validate_start_index(kwargs['start_index'])

        if 'response_format' in kwargs:
            kwargs['output_format'] = kwargs['response_format']
        if 'output_format' in kwargs:
            output_format = Client._validate_output_format(
                kwargs['output_format'])
        else:
            output_format = Client._PARSABLE_FORMAT

        return self._api_requester.post(
            self._PATH_RECORDS,
            self._build_payload(
                self.api_key,
                output_format,
                None,
                request_id,
                max_records,
                start_index
            )
        )

    def get_requests_raw(self, **kwargs) -> str:
        """
        Get raw list response
        :key output_format: Optional. Response output format.
                Supported options: JSON_FORMAT, XML_FORMAT.
                JSON_FORMAT by default
        :return: str
        :raises ConnectionError:
        :raises BulkWhoisApiError: Base class for all errors below
        :raises ResponseError: response contains an error message
        :raises ApiAuthError: Server returned 401, 402 or 403 HTTP code
        :raises BadRequestError: Server returned 400, 417 or 422 HTTP code
        :raises HttpApiError: HTTP code >= 300 and not equal to above codes
        :raises ParameterError: invalid parameter value
        """

        if self.api_key == '':
            raise EmptyApiKeyError('')

        if 'response_format' in kwargs:
            kwargs['output_format'] = kwargs['response_format']
        if 'output_format' in kwargs:
            output_format = Client._validate_output_format(
                kwargs['output_format'])
        else:
            output_format = Client._PARSABLE_FORMAT

        return self._api_requester.post(
            self._PATH_REQUESTS,
            self._build_payload(self.api_key, output_format)
        )

    @staticmethod
    def _build_payload(
            api_key,
            output_format=None,
            domains=None,
            request_id=None,
            max_records=None,
            start_index=None,
            search_type=None
    ) -> dict:
        tmp = {
            'apiKey': api_key,
            'outputFormat': output_format,
            'domains': domains,
            'requestId': request_id,
            'maxRecords': max_records,
            'startIndex': start_index,
            'searchType': search_type
        }

        payload = {}
        for k, v in tmp.items():
            if v is not None:
                payload[k] = v
        return payload

    @staticmethod
    def _validate_api_key(api_key) -> str:
        if Client._re_api_key.search(str(api_key)) is not None:
            return str(api_key)
        else:
            raise ParameterError('Invalid API key format')

    @staticmethod
    def _validate_domains(value) -> list:
        if value is None:
            raise ParameterError('Domain name list cannot be None')
        elif type(value) is list:
            if len(value) < 1:
                raise ParameterError('Domain name list cannot be empty')
            for item in value:
                if type(item) is not str:
                    raise ParameterError('Incorrect domain name value')
            return value

        raise ParameterError('Expected a list of domain names')

    @staticmethod
    def _validate_max_records(value: int) -> int:
        if type(value) is int and value > 0:
            return value

        raise ParameterError('Max records value must be greater than 0')

    @staticmethod
    def _validate_output_format(value: str):
        if type(value) is str \
                and value.lower() in [Client.JSON_FORMAT, Client.XML_FORMAT]:
            return value.lower()

        raise ParameterError(
            f'Response format must be {Client.JSON_FORMAT} '
            f'or {Client.XML_FORMAT}')

    @staticmethod
    def _validate_request_id(request_id) -> str:
        try:
            UUID(request_id)
            return request_id
        except ValueError:
            raise ParameterError('Invalid request ID format')

    @staticmethod
    def _validate_search_type(value: str):
        if type(value) is str \
                and value.lower() in \
                [Client.SEARCH_ALL, Client.SEARCH_NO_ERROR]:
            return value.lower()

        raise ParameterError(
            f'Search type type must be {Client.SEARCH_ALL} '
            f'or {Client.SEARCH_NO_ERROR}')

    @staticmethod
    def _validate_start_index(value: int) -> int:
        if type(value) is int and value > 0:
            return value

        raise ParameterError('Start index must be greater than or equal to 1')



import copy
import datetime
import sys

from whoisapi.models.base import BaseModel
from whoisapi import WhoisRecord, RegistryData

if sys.version_info < (3, 9):
    import typing


def _bool_value(values: dict, key: str) -> bool:
    if key in values and values[key]:
        return bool(values[key])
    return False


def _int_value(values: dict, key: str) -> int:
    if key in values and values[key]:
        return int(values[key])
    return 0


def _list_of_objects(values: dict, key: str, classname: str) -> list:
    r = []
    if key in values and type(values[key]) is list:
        r = [globals()[classname](x) for x in values[key]]
    return r


def _list_value(values: dict, key: str) -> list:
    if key in values and type(values[key]) is list:
        return copy.deepcopy(values[key])
    return []


def _string_value(values: dict, key: str) -> str:
    if key in values and values[key]:
        return str(values[key])
    return ''


def _timestamp2datetime(timestamp) -> datetime.datetime or None:
    if timestamp is not None:
        return datetime.datetime.utcfromtimestamp(timestamp)
    return None


class BaseModel(BaseModel):
    def __repr__(self):
        return self.__str__()


class RegistryData(RegistryData):
    def __init__(self, values):
        super().__init__(values)

        self.custom1_field_name = _string_value(values, 'customField1Name')
        self.custom1_field_value = _string_value(values, 'customField1Value')
        self.custom2_field_name = _string_value(values, 'customField2Name')
        self.custom2_field_value = _string_value(values, 'customField2Value')
        self.custom3_field_name = _string_value(values, 'customField3Name')
        self.custom3_field_value = _string_value(values, 'customField3Value')


class WhoisRecord(WhoisRecord):
    registry_data: RegistryData or None

    def __init__(self, values):
        super().__init__(values)

        self.custom1_field_name = _string_value(values, 'customField1Name')
        self.custom1_field_value = _string_value(values, 'customField1Value')
        self.custom2_field_name = _string_value(values, 'customField2Name')
        self.custom2_field_value = _string_value(values, 'customField2Value')
        self.custom3_field_name = _string_value(values, 'customField3Name')
        self.custom3_field_value = _string_value(values, 'customField3Value')

        if values is not None:
            if 'registryData' in values:
                self.registry_data = RegistryData(values['registryData'])


class BulkRequest(BaseModel):
    request_id: str
    time: datetime.datetime or None
    status: str
    total_records: int
    fetched_records: int

    def __init__(self, values):
        super().__init__()
        self.request_id = ''
        self.time = None
        self.status = ''
        self.total_records = 0
        self.fetched_records = 0

        if values is not None:
            self.request_id = _string_value(values, 'requestId')
            self.time = _timestamp2datetime(_int_value(values, 'time')/1000)
            self.status = _string_value(values, 'status')
            self.total_records = _int_value(values, 'totalRecords')
            self.fetched_records = _int_value(values, 'fetchedRecords')


class BulkWhoisRecord(BaseModel):
    domain_name: str
    domain_status: str
    whois_record_status: int
    domain_fetched_time: datetime.datetime or None
    index: int
    whois_record: WhoisRecord or None

    def __init__(self, values):
        super().__init__()
        self.domain_name = ''
        self.domain_status = ''
        self.whois_record_status = -1
        self.domain_fetched_time = None
        self.index = 0
        self.whois_record = None

        if values is not None:
            self.domain_name = _string_value(values, 'domainName')
            self.domain_status = _string_value(values, 'domainStatus')

            if 'whoisRecordStatus' in values:
                self.whois_record_status = \
                    _int_value(values, 'whoisRecordStatus')

            if 'domainFetchedTime' in values:
                self.domain_fetched_time = _timestamp2datetime(
                    int(_string_value(values, 'domainFetchedTime'))/1000
                )

            self.index = _int_value(values, 'index')

            if 'whoisRecord' in values:
                self.whois_record = WhoisRecord(values['whoisRecord'])


class ErrorMessage(BaseModel):
    code: int
    message: str

    def __init__(self, values):
        super().__init__()

        self.code = 0
        self.message = ''

        if values is not None:
            self.code = _int_value(values, 'messageCode')
            self.message = _string_value(values, 'message')


class ResponseCreate(BaseModel):
    request_id: str

    if sys.version_info < (3, 9):
        invalid_domains: typing.List[str]
    else:
        invalid_domains: [str]

    def __init__(self, values):
        super().__init__()

        self.invalid_domains = []
        self.request_id = ''

        if values is not None:
            self.invalid_domains = _list_value(values, 'invalidDomains')
            self.request_id = _string_value(values, 'requestId')


class ResponseRecords(BaseModel):
    no_data_available: bool
    request_id: str
    total_records: int
    records_left: int
    records_processed: int

    if sys.version_info < (3, 9):
        domain_list: typing.List[str]
        whois_records: typing.List[BulkWhoisRecord]
    else:
        domain_list: [str]
        whois_records: [BulkWhoisRecord]

    def __init__(self, values):
        super().__init__()

        self.no_data_available = False
        self.request_id = ''
        self.total_records = 0
        self.records_left = 0
        self.records_processed = 0
        self.domain_list = []
        self.whois_records = []

        if values is not None:
            self.no_data_available = _bool_value(values, 'noDataAvailable')
            self.request_id = _string_value(values, 'requestId')
            self.total_records = _int_value(values, 'totalRecords')
            self.records_left = _int_value(values, 'recordsLeft')
            self.records_processed = _int_value(values, 'recordsProcessed')
            self.domain_list = _list_value(values, 'domainList')

            self.whois_records = \
                _list_of_objects(values, 'whoisRecords', 'BulkWhoisRecord')


class ResponseRequests(BaseModel):
    if sys.version_info < (3, 9):
        user_requests: typing.List[BulkRequest]
    else:
        user_requests: [BulkRequest]

    def __init__(self, values):
        super().__init__()

        self.user_requests = []

        if values is not None:
            self.user_requests = \
                _list_of_objects(values, 'userRequests', 'BulkRequest')



__all__ = ['ApiAuthError', 'BadRequestError', 'BulkWhoisApiError',
           'EmptyApiKeyError', 'FileError', 'HttpApiError', 'ParameterError',
           'ResponseError', 'UnparsableApiResponseError']

from .error import ApiAuthError, BadRequestError, BulkWhoisApiError, \
    EmptyApiKeyError, FileError, HttpApiError, ParameterError, ResponseError,\
    UnparsableApiResponseError



import os
import unittest

from bulkwhoisapi import ApiAuthError, Client, FileError, ParameterError


class TestClient(unittest.TestCase):
    """
    Final integration tests without mocks.

    Active API_KEY is required.
    """

    correct_filename = 'bulk_whois_lib_test_download.csv'

    def setUp(self) -> None:
        self.client = Client(os.getenv('API_KEY'))

        self.correct_domains = ['example.com', 'example.net', 'test']
        self.correct_max_records = 1
        self.correct_request = os.getenv('BULK_WHOIS_REQUEST')

        self.incorrect_api_key = 'at_00000000000000000000000000000'
        self.incorrect_filename = 'src/does6/not257/exist5e7/8at/44all55'
        self.incorrect_request = '123'

    @classmethod
    def tearDownClass(cls) -> None:
        os.remove(cls.correct_filename)

    def test_create_correct_data(self):
        response = self.client.create_request(domains=self.correct_domains)
        self.assertEqual(len(response.invalid_domains), 1)
        self.assertIsNotNone(response.request_id)
        self.assertEqual(response.invalid_domains[0], self.correct_domains[2])

    def test_create_empty_domains(self):
        with self.assertRaises(ParameterError):
            self.client.create_request(domains=[])

    def test_download(self):
        self.client.download(filename=self.correct_filename,
                             request_id=self.correct_request)
        self.assertGreater(os.path.getsize(self.correct_filename), 0)

    def test_empty_api_key_create(self):
        with self.assertRaises(ParameterError):
            client = Client('')
            client.create_request(domains=self.correct_domains)

    def test_empty_api_key_download(self):
        with self.assertRaises(ParameterError):
            client = Client('')
            client.download(request_id=self.correct_request)

    def test_empty_api_key_get_requests(self):
        with self.assertRaises(ParameterError):
            client = Client('')
            client.get_requests()

    def test_empty_api_key_get_records(self):
        with self.assertRaises(ParameterError):
            client = Client('')
            client.get_records(request_id=self.correct_request,
                               max_records=self.correct_max_records)

    def test_empty_filename(self):
        with self.assertRaises(ParameterError):
            self.client.download(request_id=self.correct_request)

    def test_empty_max_records(self):
        with self.assertRaises(ParameterError):
            self.client.get_records(request_id=self.correct_request)

    def test_empty_request_id(self):
        with self.assertRaises(ParameterError):
            self.client.download()

    def test_get_records(self):
        response = self.client.get_records(
            request_id=self.correct_request,
            max_records=self.correct_max_records
        )
        self.assertEqual(len(response.whois_records), 1)
        self.assertIsNotNone(response.request_id)
        self.assertEqual(response.records_processed, 1)

    def test_get_requests(self):
        response = self.client.get_requests()
        self.assertGreater(len(response.user_requests), 0)
        self.assertIsNotNone(response.user_requests[0].time)

    def test_incorrect_api_key_create(self):
        with self.assertRaises(ApiAuthError):
            client = Client(self.incorrect_api_key)
            client.create_request(domains=self.correct_domains)

    def test_incorrect_api_key_get_requests(self):
        with self.assertRaises(ApiAuthError):
            client = Client(self.incorrect_api_key)
            client.get_requests()

    def test_incorrect_api_key_get_records(self):
        with self.assertRaises(ApiAuthError):
            client = Client(self.incorrect_api_key)
            client.get_records(request_id=self.correct_request,
                               max_records=self.correct_max_records)

    def test_incorrect_filename(self):
        with self.assertRaises(FileError):
            self.client.download(filename=self.incorrect_filename,
                                 request_id=self.correct_request)

    def test_incorrect_max_records(self):
        with self.assertRaises(ParameterError):
            self.client.get_records(request_id=self.correct_request,
                                    max_records='foo')

    def test_incorrect_start_index(self):
        with self.assertRaises(ParameterError):
            self.client.get_records(
                request_id=self.correct_request,
                max_records=self.correct_max_records,
                start_index='bar'
            )

    def test_incorrect_search_type(self):
        with self.assertRaises(ParameterError):
            self.client.download(
                filename=self.correct_filename,
                request_id=self.correct_request,
                search_type=123
            )

    def test_raw_create(self):
        response = self.client.create_request_raw(
            domains=self.correct_domains,
            output_format=Client.XML_FORMAT
        )
        self.assertTrue(response.startswith('<?xml'))

    def test_raw_download(self):
        response = self.client.download_raw(request_id=self.correct_request)
        self.assertTrue(response.startswith('domainName'))

    def test_raw_records(self):
        response = self.client.get_records_raw(
            request_id=self.correct_request,
            max_records=self.correct_max_records,
            output_format=Client.XML_FORMAT
        )
        self.assertTrue(response.startswith('<?xml'))

    def test_raw_requests(self):
        response = self.client.get_requests_raw(
            output_format=Client.XML_FORMAT
        )
        self.assertTrue(response.startswith('<?xml'))

    def test_output(self):
        with self.assertRaises(ParameterError):
            self.client.get_requests(response_format='yaml')


if __name__ == '__main__':
    unittest.main()



from json import JSONDecodeError, loads
from requests import request, Response

import logging

from ..exceptions.error import ApiAuthError, BadRequestError, HttpApiError
from ..version import LIBRARY_NAME, VERSION


class ApiRequester:
    __connect_timeout = 10
    __logger = logging.getLogger('api-requester')
    __user_agent = '{name}/{ver}'.format(name=LIBRARY_NAME, ver=VERSION)

    _base_url: str
    _timeout: float

    def __init__(self, **kwargs):
        """
        :param kwargs: Supported parameters:
        - base_url: (optional) API endpoint URL; str
        - timeout: (optional) API call timeout in seconds; float
        """
        self._base_url = ''
        self.timeout = 30

        if 'base_url' in kwargs:
            self.base_url = kwargs['base_url']
        if 'timeout' in kwargs:
            self.timeout = kwargs['timeout']

    @property
    def base_url(self) -> str:
        return self._base_url

    @base_url.setter
    def base_url(self, url: str):
        if url is None or len(url) <= 8 or not url.startswith('http'):
            raise ValueError('Invalid URL specified.')
        self._base_url = url

    @property
    def timeout(self) -> float:
        """API call timeout in seconds"""
        return self._timeout

    @timeout.setter
    def timeout(self, value: float):
        """API call timeout in seconds"""
        if value is not None and 1 <= value <= 60:
            self._timeout = value
        else:
            raise ValueError('Timeout value should be in [1, 60]')

    def post(self, path: str, data: dict) -> str:
        headers = {
            'User-Agent': ApiRequester.__user_agent,
            'Connection': 'close'
        }

        response = request(
            'POST',
            self.base_url + path,
            json=data,
            headers=headers,
            timeout=(ApiRequester.__connect_timeout, self.timeout)
        )

        return ApiRequester._handle_response(response)

    @staticmethod
    def _handle_response(response: Response) -> str:
        status_code = response.status_code

        try:
            parsed = loads(str(response.text))
            if 'messageCode' in parsed:
                status_code = parsed['messageCode']
        except JSONDecodeError:
            pass

        if 200 <= status_code < 300:
            return response.content.decode('UTF-8')

        if status_code in [-1, 401, 402, 403]:
            raise ApiAuthError(response.text)

        if status_code in [400, 417, 422]:
            raise BadRequestError(response.text)

        if status_code >= 300:
            raise HttpApiError(response.text)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


version_content = {}
exec(read('src/bulkwhoisapi/version.py'), version_content)

setup(
    name='bulk-whois-api',
    version=version_content['VERSION'],
    python_requires='>=3.7',
    license='MIT',
    description='Python client library for Bulk Whois API.',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.rst')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    ),
    author='WHOIS API, Inc',
    author_email='support@whoisxmlapi.com',
    url='https://github.com/whois-api-llc/bulk-whois-api-py',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Utilities',
    ],
    keywords=[
        'bulk',
        'whois',
        'api',
        'whoisxmlapi',
    ],
    install_requires=[
        'requests',
        'whois-api'
    ],
    extras_require={
        'dev': [
            'tox',
            'flake8',
        ]
    }
)
